from datetime import datetime
from typing import Callable, Union

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.database.context import DatabaseContextABC
from cpl_core.logging import LoggerABC
from cpl_core.time import TimeFormatSettings
from cpl_discord.service import DiscordBotServiceABC
from cpl_query.extension import List
from cpl_translation import TranslatePipe
from discord import Guild
from discord.ext.commands import Context

from bot_core.abc.client_utils_abc import ClientUtilsABC
from bot_core.abc.message_service_abc import MessageServiceABC
from bot_core.configuration.feature_flags_enum import FeatureFlagsEnum
from bot_core.configuration.feature_flags_settings import FeatureFlagsSettings
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.abc.user_joined_voice_channel_repository_abc import (
    UserJoinedVoiceChannelRepositoryABC,
)
from bot_data.abc.user_message_count_per_hour_repository_abc import (
    UserMessageCountPerHourRepositoryABC,
)
from bot_data.model.auto_role_rule import AutoRoleRule
from bot_data.model.server_config import ServerConfig
from bot_data.model.user import User
from bot_data.model.user_message_count_per_hour import UserMessageCountPerHour


class ClientUtilsService(ClientUtilsABC):
    def __init__(
        self,
        config: ConfigurationABC,
        logger: LoggerABC,
        time_format: TimeFormatSettings,
        bot: DiscordBotServiceABC,
        servers: ServerRepositoryABC,
        clients: ClientRepositoryABC,
        user_joined_vc: UserJoinedVoiceChannelRepositoryABC,
        umcphs: UserMessageCountPerHourRepositoryABC,
        message_service: MessageServiceABC,
        db: DatabaseContextABC,
        t: TranslatePipe,
        feature_flags: FeatureFlagsSettings,
    ):
        ClientUtilsABC.__init__(self)
        self._config = config
        self._logger = logger
        self._time_format = time_format
        self._bot = bot
        self._servers = servers
        self._umcphs = umcphs
        self._clients = clients
        self._user_joined_voice_channel = user_joined_vc
        self._message_service = message_service
        self._db = db
        self._t = t
        self._feature_flags = feature_flags

    def received_command(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.id)
        client.received_command_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def moved_user(self, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.id)
        client.moved_users_count += 1
        self._clients.update_client(client)
        self._db.save_changes()

    def moved_users(self, guild_id: int, count: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.id)
        client.moved_users_count += count
        self._clients.update_client(client)
        self._db.save_changes()

    def get_client(self, dc_ic: int, guild_id: int):
        server = self._servers.get_server_by_discord_id(guild_id)
        client = self._clients.find_client_by_discord_id_and_server_id(self._bot.user.id, server.id)
        return client

    async def check_if_bot_is_ready_yet(self) -> bool:
        if self._config.get_configuration("IS_READY") == "true":
            return True

        self._logger.debug(
            __name__,
            f'Bot is not ready yet {self._t.transform("common.errors.bot_not_ready_yet")}',
        )
        return False

    async def check_if_bot_is_ready_yet_and_respond(self, ctx: Context) -> bool:
        result = await self.check_if_bot_is_ready_yet()
        if not result:
            await self._message_service.send_ctx_msg(
                ctx,
                self._t.transform("common.errors.bot_not_ready_yet"),
                without_tracking=True,
            )

        return result

    async def presence_game(self, t_key: str):
        if not self._feature_flags.get_flag(FeatureFlagsEnum.presence):
            return

        import bot

        if self._feature_flags.get_flag(FeatureFlagsEnum.version_in_presence):
            name = f"{bot.__version__} {self._t.transform(t_key)}"
        else:
            name = self._t.transform(t_key)

        await self._bot.change_presence(activity=discord.Game(name=name))
        self._logger.info(__name__, f"Set presence {name}")

    def get_auto_complete_list(self, _l: List, current: str, select: Callable = None) -> List:
        if current != "":
            if select is None:
                select = lambda x: x

            sl = _l.select(select)
            sl = sl.where(lambda x: current in x)
            if current in sl:
                sl = sl.skip(sl.index_of(current))

            _l = _l.where(lambda x: x.name in sl)

        return _l.take(25)

    def is_message_xp_count_by_hour_higher_that_max_message_count_per_hour(
        self,
        created_at: datetime,
        user: User,
        settings: ServerConfig,
        is_reaction: bool = False,
    ) -> bool:
        umcph = None
        try:
            umcph = self._umcphs.find_user_message_count_per_hour_by_user_id_and_date(user.id, created_at)
            if umcph is None:
                self._umcphs.add_user_message_count_per_hour(
                    UserMessageCountPerHour(
                        created_at.strftime(self._time_format.date_time_format),
                        created_at.hour,
                        0,
                        user,
                    )
                )

                self._db.save_changes()

                umcph = self._umcphs.get_user_message_count_per_hour_by_user_id_and_date(user.id, created_at)
        except Exception as e:
            self._logger.error(
                __name__,
                f"Cannot add user message count per hour with id {umcph.id}",
                e,
            )
            return False

        try:
            if is_reaction:
                umcph.xp_count += settings.xp_per_reaction
            else:
                umcph.xp_count += settings.xp_per_message

            self._umcphs.update_user_message_count_per_hour(umcph)
            self._db.save_changes()
        except Exception as e:
            self._logger.error(
                __name__,
                f"Cannot update user message count per hour with id {umcph.id}",
                e,
            )
            return False

        if umcph.xp_count is None:
            return False

        return umcph.xp_count > settings.max_message_xp_per_hour

    def get_ontime_for_user(self, user: User) -> float:
        return round(
            self._user_joined_voice_channel.get_user_joined_voice_channels_by_user_id(user.id)
            .where(lambda x: x.leaved_on is not None and x.joined_on is not None)
            .sum(lambda join: (join.leaved_on - join.joined_on).total_seconds() / 3600),
            2,
        )

    async def react_to_message_by_auto_role_rule(
        self, discord_channel_id: int, discord_message_id: int, rule: AutoRoleRule, guild: discord.Guild
    ):
        try:
            guild: Guild = self._bot.guilds.where(lambda g: g == guild).single()
            channel = guild.get_channel(discord_channel_id)
            message = await channel.fetch_message(discord_message_id)
            emoji = List(discord.Emoji, guild.emojis).where(lambda x: x.name == rule.emoji_name).single()

            if emoji is None:
                self._logger.debug(__name__, f"Emoji {rule.emoji_name} not found")
                return
            await message.add_reaction(emoji)
            self._logger.debug(
                __name__,
                f"Added reaction {rule.emoji_name} to message: {discord_message_id}",
            )
        except Exception as e:
            self._logger.error(
                __name__,
                f"Cannot add reaction {rule.emoji_name} to message: {discord_message_id}",
                e,
            )

    async def check_default_role(self, member: Union[discord.User, discord.Member]):
        try:
            server = self._servers.get_server_by_discord_id(member.guild.id)
            settings: ServerConfig = self._config.get_configuration(f"ServerConfig_{server.discord_id}")

            if settings.default_role_id is None:
                return

            default_role = member.guild.get_role(settings.default_role_id)
            if default_role is None or default_role in member.roles:
                return

            await member.add_roles(default_role)

        except Exception as e:
            self._logger.error(__name__, f"Cannot check for default role for member {member.id}", e)
