import { Component, OnDestroy, OnInit } from "@angular/core";
import { ActivatedRoute, Router } from "@angular/router";
import { Queries } from "../../../../models/graphql/queries.model";
import { UserListQuery, UserWarningQuery } from "../../../../models/graphql/query.model";
import { SpinnerService } from "../../../../services/spinner/spinner.service";
import { DataService } from "../../../../services/data/data.service";
import { User } from "../../../../models/data/user.model";
import { UserDTO } from "../../../../models/auth/auth-user.dto";
import { AuthService } from "src/app/services/auth/auth.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { TranslateService } from "@ngx-translate/core";
import { Server } from "../../../../models/data/server.model";
import { forkJoin, Subject } from "rxjs";
import { takeUntil } from "rxjs/operators";
import { Table } from "primeng/table";

@Component({
  selector: "app-profile",
  templateUrl: "./profile.component.html",
  styleUrls: ["./profile.component.scss"]
})
export class ProfileComponent implements OnInit, OnDestroy {

  user: User = { createdAt: "", modifiedAt: "" };
  private server: Server = {};

  private unsubscriber = new Subject<void>();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private spinner: SpinnerService,
    private data: DataService,
    private auth: AuthService,
    private toast: ToastService,
    private translate: TranslateService
  ) {
  }

  public ngOnInit(): void {
    this.route.params.pipe(takeUntil(this.unsubscriber)).subscribe(params => {
      this.data.getServerFromRoute(this.route).then(async (server) => {
        if (!params["memberId"] || params["memberId"] == "undefined") {
          await this.router.navigate([`/server/${server.id}`]);
          return;
        }
        this.server = server;

        let authUser = await this.auth.getLoggedInUser();
        this.spinner.showSpinner();
        let user: UserDTO | null = authUser?.users?.find(u => u.server == server.id) ?? null;
        if (!user || user?.id != params["memberId"] && !user?.isModerator) {
          this.toast.error(this.translate.instant("view.server.profile.permission_denied"), this.translate.instant("view.server.profile.permission_denied_d"));
          this.spinner.hideSpinner();
          await this.router.navigate(["/server", server.id]);
          return;
        }

        this.data.query<UserListQuery>(Queries.userProfile, {
            serverId: this.server.id,
            userId: params["memberId"]
          }
        ).subscribe(users => {
          if (!users.users[0]) {
            this.router.navigate([`/server/${server.id}`]);
          }
          this.user = users.users[0];

          this.data.query<UserWarningQuery>(Queries.userProfileWarnings, {
              serverId: this.server.id,
              userId: this.user.id
            },
            (data: UserListQuery) => {
              return data.users[0];
            }
          ).subscribe(result => {
            this.user.userWarningCount = result.userWarningCount;
            this.user.userWarnings = result.userWarnings;
            console.log(result);

            this.spinner.hideSpinner();
          });
        });
      });
    });
  }

  public ngOnDestroy(): void {
    this.unsubscriber.next();
    this.unsubscriber.complete();
  }

  public onBeforeToggle(event: Event, collapsed: boolean) {
    const filterUser = (x: { users: User[] }) => {
      const users = x.users ?? [];
      return users[0];
    };

    if (collapsed) {
      this.spinner.showSpinner();
      forkJoin([
        this.data.query<User>(Queries.userProfileAchievements, {
            serverId: this.server.id,
            userId: this.user.id
          },
          filterUser
        ),
        this.data.query<User>(Queries.userProfileVoiceChannelJoins, {
            serverId: this.server.id,
            userId: this.user.id
          },
          filterUser
        ),
        this.data.query<User>(Queries.userProfileGameserverJoins, {
            serverId: this.server.id,
            userId: this.user.id
          },
          filterUser
        )
      ]).subscribe(data => {
        this.user.achievementCount = data[0].achievementCount;
        this.user.achievements = data[0].achievements;

        this.user.joinedVoiceChannelCount = data[1].joinedVoiceChannelCount;
        this.user.joinedVoiceChannels = data[1].joinedVoiceChannels;

        this.user.userJoinedGameServerCount = data[2].userJoinedGameServerCount;
        this.user.userJoinedGameServers = data[2].userJoinedGameServers;
        this.spinner.hideSpinner();
      });
      return;
    }
    this.user.achievementCount = 0;
    this.user.achievements = [];

    this.user.userJoinedGameServerCount = 0;
    this.user.userJoinedGameServers = [];

    this.user.joinedVoiceChannelCount = 0;
    this.user.joinedVoiceChannels = [];
  }

  addNewUserWarning(table: Table) {
  }

  deleteUserWarning(index: number) {
  }

  editSaveUserWarning(value: any, index: number) {
  }

  editCancelUserWarning(index: number) {
  }

  protected readonly visualViewport = visualViewport;
}
